;;
;;	ENTITY MANAGER
;;
.include "entity_manager.h.s"
.include "cpct_functions.h.s"
.include "sys/render_system.h.s"
.include "man_game.h.s"
.include "cpctelera.h.s"

DefineEntityArray 	_star_array, 	20					; Array de estrellas
DefineEntityArray	_nave_array, 	 6					; Array de la nave y los disparos 
DefineEntityArray   _enemy_array, 	30 					; Array de los enmigos 
_num_entities :: 	.db 0								; Contador de entidades creadas
_last_entity :: 	.dw 0x0000							; Direccion de la ultima entidad 
_last_elem_ptr::	.dw _star_array						; Puntero a la primera entidad 
_max_entities:		.db 0								; Numero maximo de entidades que tiene el array
_lifes_of_planet:: 	.db 3								; Vidas que tiene el planeta tierra

.globl _nada_sp

entity_man_reset_entities::
	call entity_man_getEnemyArray_IX
	ld hl, #entity_man_set4destroy
	call entity_man_forAll
	call entity_man_getEnemyArray_IX
	ld hl, #entity_man_setEnemyArray_IX
	call entity_man_update

	call entity_man_getStarsArray_IX
	ld hl, #entity_man_set4destroy
	call entity_man_forAll
	call entity_man_getStarsArray_IX
	ld hl, #entity_man_setStarsArray_IX
	call entity_man_update

	call entity_man_getNaveArray_IX
	ld hl, #entity_man_set4destroy
	call entity_man_forAll
	call entity_man_getNaveArray_IX
	ld hl, #entity_man_setNaveArray_IX
	call entity_man_update

	call entity_man_reset_lifes

	ret
;--------------------------------------------------------------
;	GETTER Y SETTER
;--------------------------------------------------------------

; Funcion geter que coloca la posicion de _star_array en IX
entity_man_getStarsArray_IX::
   	ld    	ix, #_star_array
   	ld 		a, (_max_entities_star_array)		
   	ld 		(_max_entities), a
   	ld 		a, (_num_entities_star_array)
   	ld 		(_num_entities), a
   	ld 		bc, (_last_elem_ptr_star_array)
   	ld 		(_last_elem_ptr), bc
	ret
; Funcion geter que coloca la posicion de _star_array en IX
entity_man_setStarsArray_IX::
	ld 		a, (_num_entities)
   	ld 		(_num_entities_star_array), a
   	ld 		(_last_elem_ptr_star_array), hl
	ret
;-------------------------------------------------------------------------
; Funcion geter que coloca la posicion de _nave_array en IX
entity_man_getNaveArray_IX::
   	ld    	ix, #_nave_array
   	ld 		a, (_max_entities_nave_array)		
   	ld 		(_max_entities), a
   	ld 		a, (_num_entities_nave_array)
   	ld 		(_num_entities), a
   	ld 		bc, (_last_elem_ptr_nave_array)
   	ld 		(_last_elem_ptr), bc
	ret
; Funcion geter que coloca la posicion de _star_array en IX
entity_man_setNaveArray_IX::
	ld 		a, (_num_entities)
   	ld 		(_num_entities_nave_array), a
   	ld 		(_last_elem_ptr_nave_array), hl
	ret
;-------------------------------------------------------------------------
; Funcion geter que coloca la posicion de _star_array en IX
entity_man_getEnemyArray_IX::
   	ld    	ix, #_enemy_array
   	ld 		a, (_max_entities_enemy_array)		
   	ld 		(_max_entities), a
   	ld 		a, (_num_entities_enemy_array)
   	ld 		(_num_entities), a
   	ld 		bc, (_last_elem_ptr_enemy_array)
   	ld 		(_last_elem_ptr), bc
	ret
; Funcion geter que coloca la posicion de _star_array en IX
entity_man_setEnemyArray_IX::
	ld 		a, (_num_entities)
   	ld 		(_num_entities_enemy_array), a
   	ld 		(_last_elem_ptr_enemy_array), hl
	ret
;-------------------------------------------------------------------------

; Funcion geter que coloca el valor que hay en _num_entities en A
entity_man_getNumEntities_A::
	ld    	a, (_num_entities)
	ret

; Funcion geter que coloca la posicion de entity_size en BC
entity_man_getNumEntities_BC::
	ld 		bc, #entity_size
	ret

; Funcion para darte la ultima entidad del array
entity_man_getLastEntity_IX::
	ld 		hl, (_last_elem_ptr)		
	ld 		de, #-entity_size 		
	add		hl, de 	
	ld 		(_last_entity), hl
	ld 		ix, (_last_entity)
	ret

; Funcion que nos dice si quedan espacio en memoria para crear nuevas entidades 
;	OUTPUT
;		A: Nos devuelve el numero de entidades que podemos crear 
entity_man_getFreeSpace_A::
	ld 		a, (_num_entities) 
	neg
	ld 		hl, #_max_entities
	add		(hl)
	ret


;--------------------------------------------------------------
;	FUNCIONES 
;--------------------------------------------------------------

; Funcion para inicializar el  juego 
entity_man_init::
	xor 	a 						; Colocamos un 0 en el registro A
	ld 		(_num_entities), a 		; Ponemos a 0 la variable _num_entities

	ld 		hl, #_star_array 		; Colocamos en principio del Array en HL
	ld 		(_last_elem_ptr), hl 	; Indicamos que  el _last_elem_ptr esta en el principio del Array
	ret


; Funcion que nos sirve para introducir lo que  hay en la posicion HL en nuestro array de entidades
; INPUT
; 	HL: Puentero a los bytes de inicializacion
entity_man_create::

	ld		de, (_last_elem_ptr)	; \
	ld		bc, #entity_size 		; | Copia la entidad en la ultima posicion del array
	ldir 							; /

	ld 		a, (_num_entities)		; \
	inc 	a 						; | Aumentamos el numero de entidades que tenemos 
	ld 		(_num_entities), a 		; /

	ld 		hl, (_last_elem_ptr) 	; \
	ld 		bc, #entity_size 		; |
	add		hl, bc 					; | Colocamos el puntero de la ultima entidad en la nueva posicion
	ld 		(_last_elem_ptr), hl 	; /

	ret


; Funcion que marca para destruir una entidad 
; INPUT
; 	IX: La entidad que vamos a poner para destruir 
entity_man_set4destroy::
	ld 		a, e_state(ix)
	or 		#entity_dead
	ld 		e_state(ix), a
	ret

; Funcion que marca para destruir una entidad 
; INPUT
; 	IY: La entidad que vamos a poner para destruir 
entity_man_set4destroy_IY::
	ld 		a, e_state(iy)
	or 		#entity_dead
	ld 		e_state(iy), a
	ret


; Funcion que nos sirve para introducir lo que  hay en la posicion HL en nuestro array de entidades
;	IMPUT
;		IX: Tiene que estar la  direccion de la entidad 
entity_man_destroy::

	ld 		hl, (_last_elem_ptr)				; Colocamos en hl el puntero a la  ultima entidad
	ld 		de, #-entity_size 					; Le rectamos El tamaño de una entidad 
	add		hl, de 								; dejamos en hl la direccion de memoria de la ultima entidad del array
	ld 		(_last_entity), hl

	ld__a_ixl									; \
	sub		l 									; |
	jr		nz, copiar 							; | Lo unico que se hace aquí es comprobar si la direccion de la entidad 
	ld__a_ixh 									; | a borar es igual a la ultima entidad 
	sub		h 									; |
	jr		z, no_copiar 						; /	
	copiar:								
		ld__d_ixh								; \
		ld__e_ixl								; | si no es igual se copia la ultima entidad en la entidad a eliminar
		ld 	bc, #entity_size					; /
		ldir

	no_copiar:
		ld 		ix, (_last_entity)  			;			
		ld 		e_state(ix), #entity_invalid	; Se coloca como invalida la ultima entidad 
		ld  	hl, (_last_entity) 	    		; Se actualiza la ultima posicion del array
		ld 		(_last_elem_ptr), hl 			;
		ld 		a, (_num_entities)				; \
		dec 	a   							; | Se decrementa el numero de entidades
		ld 		(_num_entities), a 				; /

	ret


; Funcion para destruir las entidades marcadas para destruir
; 	INPUT:
; 		IX: El array que queremos actualizar 
entity_man_update::

	ld 		(_function2), hl
	jr		cond

	destroy:
		call 	entity_man_destroy 		; Elimiamos la entidad 
		_function2 = .+1
		call _function2

	cond:
		ld 		a, e_state(ix)			; \
		xor		#entity_invalid 		; | Comprobamos que no se sea una entidad invalida 
		ret 	z 						; /

		ld 		a, e_state(ix) 			; \
		and		#entity_dead 			; | Si la entidad esta marcada para eliminar la mandamos a eliminar
		jr 		nz, destroy  			; /

	no_destroy:
		ld 		bc, #entity_size 		; \
		add 	ix, bc 					; | Pasamos a la siguiente entidad 
		jr		cond 					; /

	ret


; Funcion que ejecuta otra funcion sobre todaslas entidades 
; INPUT
; 	HL: La direccion de memoria de la funcion que queremos aplicar a todas las entidades
;	IX: Se le pasa el array que se va a utilizar
entity_man_forAll::
	ld 		(_function), hl

	next_entity:
		ld 		a, e_state(ix)			; \
		xor		#entity_invalid 		; | Comprobamos que no se sea una entidad invalida 
		ret 	z 						; /

		_function = .+1
		call _function 					; Llamamos a la funcion que queremos aplicar a la entidad 

		ld 		bc, #entity_size 		; \
		add 	ix, bc 					; | Pasamos a la siguiente entidad 
		jr		next_entity				; /

entity_man_reset_lifes::
	ld 		a, #3
	ld 		(_lifes_of_planet), a
	ret
		
entity_man_lose_a_life::
	ld 		a, (_lifes_of_planet)
	add		#0
	ret 	z
	add 	#-1
	ld 		(_lifes_of_planet), a
	and 	#2
	jr 		nz, cont
	call 	render_lose_live2
	ret
	cont:
	call 	render_lose_live1
	ret	

entity_man_check_lifes::
	ld 		a, (_lifes_of_planet)
	add 	#0
	ret 	nz
	call 	gameOver
	ret