;;
;; SYSTEMA DE FISICAS
;;
.include "man/entity_manager.h.s"
.include "cpctelera.h.s"
.include "physics_system.h.s"

screen_width = 80 ; 80 bytes de ancho de pantalla
screen_height = 174
screen_width_init = 5


.globl _bala_sp
.globl _bala_pal
;_x, _y, _w, _h _vx, _vy, _color, _prevX, _prevY, _state
DefineEntity bullet, 0, 0, 2, 4, 5, 0, _bala_sp, 0, 0, #entity_bullet

;--------------------------------------------------------------
;	FUNCIONES 
;--------------------------------------------------------------


;Funcion para actualizar una entidad
physics_sys_update_one::

	ld		a, #screen_width + 1 	; Calculamos el tamaño maximo 
	sub		e_w(ix)					; al cual puede llegar mi entidad
	ld 		c, a

	ld 		a, e_x(ix)				; \
	add		e_vx(ix)				; | 
	cp 		#screen_width_init 		; |
	jr 		c, invalid_x			; |
	cp 		c 						; | Comprobamos si la nueva posicion de x es valida 
	jr 		c, valid_x				; /

	invalid_x:
		ld 		a, e_state(ix)				; \
		and 	#entity_player				; | Si es el jugador no lo ponemos para destruir 
		jr		nz, part_y 					; /

		ld 		a, e_state(ix)
		and 	#entity_enemy
		jr 		z, go_die
		call 	entity_man_lose_a_life

		go_die:
		call 	entity_man_set4destroy		;   Si no la marcamos para destruir

	valid_x:
		ld		e_x(ix), a 				; Se carga en la posicion de la entidad el nuevo valor 

		ld		a, #screen_height + 1 	; Calculamos el tamaño maximo 
		sub		e_h(ix)					; al cual puede llegar mi entidad
		ld 		c, a

	part_y: 
		ld 		a, e_y(ix)				; \
		add		e_vy(ix)				; | 
		cp 		c 						; | Comprobamos si la nueva posicion de y es valida 
		jr 		c, valid_y				; /

	invalid_y:
		ld 		a, e_state(ix) 			; \
		and 	#entity_player			; | Si es el jugador no lo ponemos para destruir 
		ret		nz						; /
		call 	entity_man_set4destroy		;   Si no la marcamos para destruir

	valid_y:
		ld		e_y(ix), a 			; Se carga en la posicion de la entidad el nuevo valor 
		ret


; Funcion para actualizar las Fisiscas de las estrellas
physics_sys_update::
	
	ld      hl, #physics_sys_update_one 	
    call 	entity_man_getStarsArray_IX
    call    entity_man_forAll
    ret

; Funcion para actualizar las Fisiscas de la nave
physics_sys_update_Nave::
	
	ld      hl, #physics_sys_update_one 
	call 	entity_man_getNaveArray_IX
    call    entity_man_forAll
    ret


;; Funcion para actualizar las fisicas del enemigo
physics_sys_update_Enemigo::
	
	ld      hl, #physics_sys_update_one 
	call 	entity_man_getEnemyArray_IX
    call    entity_man_forAll
    ret

	
;DESCRIPCION:
;;Funcion que recibe la nave siempre en la primera posicion del array de entidades 
;;y actualiza sus fisicas en función de las teclas pulsadas. Tambien llama a la funcion
;;de disparo
;PRECONDICIONES:
;;   La nave siempre se encontrara en la primera posicion del array
;MODIFICA: IX, HL
physics_ship_keyboard::
	call  	entity_man_getNaveArray_IX
	ld 		e_vx(ix), #0 ;\ Cargamos un 0 en las velocidades de la nave para
	ld 		e_vy(ix), #0 ;/ que cuando no pulsemos nada la nave no se mueva
	
	call 	cpct_scanKeyboard_f_asm
	ld 		hl, #Key_W
	call 	cpct_isKeyPressed_asm
	jr z, 	arriba_no_pulsada ; si no se ha pulsado -> flag z = 1

	arriba_pulsada: ; no saltamos, la tecla se ha pulsado
		ld 	e_vy(ix), #-7
	
	arriba_no_pulsada: ; hemos saltado
		ld 		hl, #Key_S
		call 	cpct_isKeyPressed_asm
		jr 		z, abajo_no_pulsado
	
	abajo_pulsado:
		ld 		e_vy(ix), #7

	abajo_no_pulsado:
		ld 		hl, #Key_A
		call 	cpct_isKeyPressed_asm
		jr 		z, izquierda_no_pulsada

	izquierda_pulsada:
		ld 		e_vx(ix), #-2
	
	izquierda_no_pulsada:
      	ld 		hl, #Key_D
      	call 	cpct_isKeyPressed_asm
      	jr 		z, derecha_no_pulsada
   
   	derecha_pulsada:
      	ld 		e_vx(ix), #2
   
   	derecha_no_pulsada:
      	ld 		hl, #Key_Space
      	call 	cpct_isKeyPressed_asm
      	jr 		z, espacio_no_pulsado

      
    espacio_pulsado:
	  	call 	sys_physics_bullet
		
	espacio_no_pulsado:
		ret


	ret

; Funcion para crear una bala 
sys_physics_bullet::
	
	call 	entity_man_getNaveArray_IX
	call 	entity_man_getFreeSpace_A
	ret 	z
		
	ld 		hl, #bullet  					;
    call 	entity_man_create					; Creas una Bala y la guardas en la ultima posicion
    call 	entity_man_setNaveArray_IX

    ld 		b, e_x(ix)						; | Guardas en BC los valores de X e Y de la nave 
    ld 		c, e_y(ix)						; /

    call 	entity_man_getLastEntity_IX

    ld  	a, b 							; \
    add 	a, #4							; |
    ld 		e_x(ix), a 						; |
    										; | Colocamos la Bala un poco mas adelante de la nave 
    ld  	a, c 							; |
    add 	a, #3							; |
    ld 		e_y(ix), a 						; /

	ret
